@@ -63,6 +63,64 @@ test_that("use_cassette: match_requests_on - body works w/ crul", {
   expect_equal(aa$name, "testing3")
   expect_equal(aa$match_requests_on, "body")
 
+  ### matchers: host only (note how query is ignored)
+  # run it
+  aa <- use_cassette(name = "testing_host1", {
+    res <- HttpClient$new(url = "https://httpbin.org")$get(query = list(b=99999))
+  }, match_requests_on = "host")
+  # run it again
+  bb <- use_cassette(name = "testing_host1", {
+    res2 <- HttpClient$new(url = "https://httpbin.org")$get(query = list(a=5))
+  }, match_requests_on = "host")
+  # the recorded_at time doesn't change
+  # - that is, the request matched and the recorded response in aa
+  # - was used
+  expect_identical(recorded_at(aa), recorded_at(bb))
+  expect_is(aa, "Cassette")
+  expect_is(aa$name, "character")
+  expect_equal(aa$name, "testing_host1")
+  expect_equal(aa$match_requests_on, "host")
+
+  ### matchers: path only (note how host and query differences are ignored)
+  # run it
+  aa <- use_cassette(name = "testing_path1", {
+    res <- HttpClient$new("https://scottchamberlain.info")$get(
+      "about", query = list(b=99999))
+  }, match_requests_on = "path")
+  # run it again
+  bb <- use_cassette(name = "testing_path1", {
+    res2 <- HttpClient$new("https://ropensci.org")$get(
+      "about", query = list(a=5))
+  }, match_requests_on = "path")
+  # the recorded_at time doesn't change
+  # - that is, the request matched and the recorded response in aa
+  # - was used
+  expect_identical(recorded_at(aa), recorded_at(bb))
+  expect_is(aa, "Cassette")
+  expect_is(aa$name, "character")
+  expect_equal(aa$name, "testing_path1")
+  expect_equal(aa$match_requests_on, "path")
+
+  ### matchers: host and path only (notice how HTTP method and query are ignored)
+  # run it
+  aa <- use_cassette(name = "testing_host_path", {
+    res <- HttpClient$new(url = "https://ropensci.org")$get(
+      "about", query = list(b=99999))
+  }, match_requests_on = c("host", "path"))
+  # run it again
+  bb <- use_cassette(name = "testing_host_path", {
+    res2 <- HttpClient$new(url = "https://ropensci.org")$post(
+      "about", query = list(a=5))
+  }, match_requests_on = c("host", "path"))
+  # the recorded_at time doesn't change
+  # - that is, the request matched and the recorded response in aa
+  # - was used
+  expect_identical(recorded_at(aa), recorded_at(bb))
+  expect_is(aa, "Cassette")
+  expect_is(aa$name, "character")
+  expect_equal(aa$name, "testing_host_path")
+  expect_equal(aa$match_requests_on, c("host", "path"))
+
   # cleanup
   unlink(mydir, recursive = TRUE)
 })
@@ -135,6 +193,24 @@ test_that("use_cassette: match_requests_on - body works w/ httr", {
   expect_equal(aa$match_requests_on, "body")
   expect_equal(bb$match_requests_on, "body")
 
+  ### matchers: host and path only (notice how HTTP method and query are ignored)
+  # run it
+  aa <- use_cassette(name = "testing_httr_host_path", {
+    res <- GET("https://ropensci.org/about", query = list(b=99999))
+  }, match_requests_on = c("host", "path"))
+  # run it again
+  bb <- use_cassette(name = "testing_httr_host_path", {
+    res2 <- POST("https://ropensci.org/about", query = list(a=5))
+  }, match_requests_on = c("host", "path"))
+  # the recorded_at time doesn't change
+  # - that is, the request matched and the recorded response in aa
+  # - was used
+  expect_identical(recorded_at(aa), recorded_at(bb))
+  expect_is(aa, "Cassette")
+  expect_is(aa$name, "character")
+  expect_equal(aa$name, "testing_httr_host_path")
+  expect_equal(aa$match_requests_on, c("host", "path"))
+
   # cleanup
   unlink(mydir, recursive = TRUE)
 })
